@@ -113,7 +113,7 @@ impl CollectionBuilder {
         store: &impl Storelike,
         for_agent: Option<&str>,
     ) -> AtomicResult<Collection> {
-        Collection::new_with_members(store, self, for_agent)
+        Collection::collect_members(store, self, for_agent)
     }
 }
 
@@ -153,6 +153,7 @@ pub struct Collection {
 }
 
 /// Sorts a vector or resources by some property.
+#[tracing::instrument]
 fn sort_resources(
     mut resources: ResourceCollection,
     sort_by: &str,
@@ -183,7 +184,8 @@ impl Collection {
     /// Constructs a Collection, which is a paginated list of items with some sorting applied.
     /// Gets the required data from the store.
     /// Applies sorting settings.
-    pub fn new_with_members(
+    #[tracing::instrument(skip(store))]
+    pub fn collect_members(
         store: &impl Storelike,
         collection_builder: crate::collections::CollectionBuilder,
         for_agent: Option<&str>,
@@ -192,6 +194,7 @@ impl Collection {
             return Err("Page size must be greater than 0".into());
         }
         // Execute the TPF query, get all the subjects.
+        // Note that these are not yet authorized.
         let atoms = store.tpf(
             None,
             collection_builder.property.as_deref(),
@@ -199,32 +202,44 @@ impl Collection {
             collection_builder.include_external,
         )?;
         // Remove duplicate subjects
-        let mut subjects: Vec<String> = atoms
+        let mut subjects_deduplicated: Vec<String> = atoms
             .iter()
             .map(|atom| atom.subject.clone())
             .collect::<std::collections::HashSet<String>>()
             .into_iter()
             .collect();
 
-        // Sort by subject, by default
-        subjects.sort();
+        // Sort by subject, better than no sorting
+        subjects_deduplicated.sort();
 
+        // WARNING: Entering expensive loop!
+        // This is needed for sorting, authorization and including nested resources.
+        // It could be skipped if there is no authorization and sorting requirement.
         let mut resources = Vec::new();
-        // If sorting is required or the nested resoureces are asked, we need to fetch all resources from the store.
-        if collection_builder.sort_by.is_some() || collection_builder.include_nested {
-            for subject in subjects.iter() {
-                // These nested resources are not fully calculated - they will be presented as -is
-                let resource = store.get_resource_extended(subject, true, for_agent)?;
-                resources.push(resource)
-            }
-            if let Some(sort) = &collection_builder.sort_by {
-                resources = sort_resources(resources, sort, collection_builder.sort_desc);
-                subjects.clear();
-                for r in resources.iter() {
-                    subjects.push(r.get_subject().clone())
+        for subject in subjects_deduplicated.iter() {
+            // These nested resources are not fully calculated - they will be presented as -is
+            match store.get_resource_extended(subject, true, for_agent) {
+                Ok(resource) => {
+                    resources.push(resource);
                 }
+                Err(e) => match e.error_type {
+                    crate::AtomicErrorType::NotFoundError => {}
+                    crate::AtomicErrorType::UnauthorizedError => {}
+                    crate::AtomicErrorType::OtherError => {
+                        return Err(
+                            format!("Error when getting resource in collection: {}", e).into()
+                        )
+                    }
+                },
             }
         }
+        if let Some(sort) = &collection_builder.sort_by {
+            resources = sort_resources(resources, sort, collection_builder.sort_desc);
+        }
+        let mut subjects = Vec::new();
+        for r in resources.iter() {
+            subjects.push(r.get_subject().clone())
+        }
         let mut all_pages: Vec<Vec<String>> = Vec::new();
         let mut all_pages_nested: Vec<Vec<Resource>> = Vec::new();
         let mut page: Vec<String> = Vec::new();
@@ -368,10 +383,12 @@ impl Collection {
 /// Builds a collection from query params and the passed Collection resource.
 /// The query params are used to override the stored Collection resource properties.
 /// This also sets defaults for Collection properties when fields are missing
-pub fn construct_collection(
+#[tracing::instrument(skip(store, query_params))]
+pub fn construct_collection_from_params(
     store: &impl Storelike,
     query_params: url::form_urlencoded::Parse,
     resource: &mut Resource,
+    for_agent: Option<&str>,
 ) -> AtomicResult<Resource> {
     let mut sort_by = None;
     let mut sort_desc = false;
@@ -423,7 +440,7 @@ pub fn construct_collection(
         include_nested,
         include_external,
     };
-    let collection = Collection::new_with_members(store, collection_builder, None)?;
+    let collection = Collection::collect_members(store, collection_builder, for_agent)?;
     collection.add_to_resource(resource, store)
 }
 
@@ -499,7 +516,7 @@ mod test {
             include_nested: false,
             include_external: false,
         };
-        let collection = Collection::new_with_members(&store, collection_builder, None).unwrap();
+        let collection = Collection::collect_members(&store, collection_builder, None).unwrap();
         assert!(collection.members.contains(&urls::PROPERTY.into()));
     }
 
@@ -519,7 +536,7 @@ mod test {
             include_nested: false,
             include_external: false,
         };
-        let collection = Collection::new_with_members(&store, collection_builder, None).unwrap();
+        let collection = Collection::collect_members(&store, collection_builder, None).unwrap();
         assert!(collection.members.contains(&urls::PROPERTY.into()));
 
         let resource_collection = &collection.to_resource(&store).unwrap();
@@ -545,7 +562,7 @@ mod test {
             include_nested: true,
             include_external: false,
         };
-        let collection = Collection::new_with_members(&store, collection_builder, None).unwrap();
+        let collection = Collection::collect_members(&store, collection_builder, None).unwrap();
         let first_resource = &collection.members_nested.clone().unwrap()[0];
         assert!(first_resource.get_subject().contains("Agent"));
 
