@@ -5,8 +5,8 @@ use std::collections::{HashMap, HashSet};
 use urls::{SET, SIGNER};
 
 use crate::{
-    datatype::DataType, datetime_helpers, errors::AtomicResult, resources::PropVals, urls, Atom,
-    AtomicError, Resource, Storelike, Value,
+    datatype::DataType, datetime_helpers, errors::AtomicResult, hierarchy, resources::PropVals,
+    urls, Atom, Resource, Storelike, Value,
 };
 
 /// Contains two resources. The first is the Resource representation of the applied Commits.
@@ -14,10 +14,10 @@ use crate::{
 /// The third is the Old resource.
 #[derive(Clone, Debug)]
 pub struct CommitResponse {
-    pub commit: Resource,
+    pub commit_resource: Resource,
     pub resource_new: Option<Resource>,
     pub resource_old: Resource,
-    pub full_commit: Commit,
+    pub commit_struct: Commit,
 }
 
 /// A Commit is a set of changes to a Resource.
@@ -65,6 +65,7 @@ impl Commit {
     /// Allows for control over which validations should be performed.
     /// Returns the generated Commit, the old Resource and the new Resource.
     /// TODO: Should check if the Agent has the correct rights.
+    #[tracing::instrument(skip(store))]
     pub fn apply_opts(
         &self,
         store: &impl Storelike,
@@ -76,6 +77,7 @@ impl Commit {
     ) -> AtomicResult<CommitResponse> {
         let subject_url = url::Url::parse(&self.subject)
             .map_err(|e| format!("Subject '{}' is not a URL. {}", &self.subject, e))?;
+
         if subject_url.query().is_some() {
             return Err("Subject URL cannot have query parameters".into());
         }
@@ -123,11 +125,7 @@ impl Commit {
 
         if validate_rights {
             if is_new {
-                if !crate::hierarchy::check_write(store, &resource_new, &self.signer)? {
-                    return Err(AtomicError::unauthorized(
-                        format!("Agent {} is not permitted to create {}. There should be a write right referring to this Agent in this Resource or its parent.",
-                    &self.signer, self.subject)));
-                }
+                hierarchy::check_write(store, &resource_new, &self.signer)?;
             } else {
                 // Set a parent only if the rights checks are to be validated.
                 // If there is no explicit parent set on the previous resource, use a default.
@@ -141,16 +139,31 @@ impl Commit {
                     )?;
                 }
                 // This should use the _old_ resource, no the new one, as the new one might maliciously give itself write rights.
-                if !crate::hierarchy::check_write(store, &resource_old, &self.signer)? {
-                    return Err(AtomicError::unauthorized(format!("Agent {} is not permitted to edit {}. There should be a write right referring to this Agent in this Resource or its parent.",
-                    &self.signer, self.subject)));
-                }
+                hierarchy::check_write(store, &resource_old, &self.signer)?;
             }
         };
         // Check if all required props are there
         if validate_schema {
             resource_new.check_required_props(store)?;
         }
+
+        // TODO: before_apply_commit hooks plugins
+        // This is where users should extend commits
+        for class in resource_new.get_classes(store)? {
+            match class.subject.as_str() {
+                urls::COMMIT => return Err("Commits can not be edited or created directly.".into()),
+                urls::INVITE => {
+                    // Check if the creator has rights to invite people (= write) to the target resource
+                    let target = resource_new
+                        .get(urls::TARGET)
+                        .map_err(|_e| "Invite does not have required Target attribute")?;
+                    let target_resource = store.get_resource(&target.to_string())?;
+                    hierarchy::check_write(store, &target_resource, &self.signer)?;
+                }
+                _other => {}
+            };
+        }
+
         // If a Destroy field is found, remove the resource and return early
         // TODO: Should we remove the existing commits too? Probably.
         if let Some(destroy) = self.destroy {
@@ -161,8 +174,8 @@ impl Commit {
                 return Ok(CommitResponse {
                     resource_new: None,
                     resource_old,
-                    commit: commit_resource,
-                    full_commit: self.clone(),
+                    commit_resource,
+                    commit_struct: self.clone(),
                 });
             }
         }
@@ -175,13 +188,14 @@ impl Commit {
         Ok(CommitResponse {
             resource_new: Some(resource_new),
             resource_old,
-            commit: commit_resource,
-            full_commit: self.clone(),
+            commit_resource,
+            commit_struct: self.clone(),
         })
     }
 
     /// Updates the values in the Resource according to the `set`, `remove` and `destroy` attributes in the Commit.
     /// Optionally also updates the index in the Store.
+    #[tracing::instrument(skip(store))]
     pub fn apply_changes(
         &self,
         mut resource: Resource,
@@ -228,6 +242,7 @@ impl Commit {
     }
 
     /// Converts a Resource of a Commit into a Commit
+    #[tracing::instrument]
     pub fn from_resource(resource: Resource) -> AtomicResult<Commit> {
         let subject = resource.get(urls::SUBJECT)?.to_string();
         let created_at = resource.get(urls::CREATED_AT)?.to_int()?;
@@ -262,6 +277,7 @@ impl Commit {
     /// Converts the Commit into a Resource with Atomic Values.
     /// Creates an identifier using the base_url
     /// Works for both Signed and Unsigned Commits
+    #[tracing::instrument(skip(store))]
     pub fn into_resource(self, store: &impl Storelike) -> AtomicResult<Resource> {
         let commit_subject = match self.signature.as_ref() {
             Some(sig) => format!("{}/commits/{}", store.get_base_url(), sig),
@@ -323,6 +339,7 @@ impl Commit {
 
     /// Generates a deterministic serialized JSON-AD representation of the Commit.
     /// Removes the signature from the object before serializing, since this function is used to check if the signature is correct.
+    #[tracing::instrument(skip(store))]
     pub fn serialize_deterministically_json_ad(
         &self,
         store: &impl Storelike,
@@ -396,6 +413,7 @@ impl CommitBuilder {
 }
 
 /// Signs a CommitBuilder at a specific unix timestamp.
+#[tracing::instrument(skip(store))]
 fn sign_at(
     commitbuilder: CommitBuilder,
     agent: &crate::agents::Agent,
@@ -427,6 +445,7 @@ fn sign_at(
 }
 
 /// Signs a string using a base64 encoded ed25519 private key. Outputs a base64 encoded ed25519 signature.
+#[tracing::instrument]
 pub fn sign_message(message: &str, private_key: &str, public_key: &str) -> AtomicResult<String> {
     let private_key_bytes = base64::decode(private_key.to_string()).map_err(|e| {
         format!(
