@@ -8,6 +8,7 @@ use std::{
 
 use crate::{
     datatype::DataType,
+    endpoints::{default_endpoints, Endpoint},
     errors::{AtomicError, AtomicResult},
     resources::PropVals,
     storelike::{ResourceCollection, Storelike},
@@ -33,6 +34,8 @@ pub struct Db {
     index_vals: sled::Tree,
     /// The base_url is the domain where the db will be hosted, e.g. http://localhost/
     base_url: String,
+    /// Endpoints are checked whenever a resource is requested. They calculate (some properties of) the resource and return it.
+    endpoints: Vec<Endpoint>,
 }
 
 impl Db {
@@ -49,6 +52,7 @@ impl Db {
             resources,
             index_vals,
             base_url,
+            endpoints: default_endpoints(),
         };
         crate::populate::populate_base_models(&store)
             .map_err(|e| format!("Failed to populate base models. {}", e))?;
@@ -56,6 +60,7 @@ impl Db {
     }
 
     /// Internal method for fetching Resource data.
+    #[tracing::instrument(skip(self))]
     fn set_propvals(&self, subject: &str, propvals: &PropVals) -> AtomicResult<()> {
         let resource_bin = bincode::serialize(propvals)?;
         let subject_bin = bincode::serialize(subject)?;
@@ -65,6 +70,7 @@ impl Db {
 
     /// Finds resource by Subject, return PropVals HashMap
     /// Deals with the binary API of Sled
+    #[tracing::instrument(skip(self))]
     fn get_propvals(&self, subject: &str) -> AtomicResult<PropVals> {
         let subject_binary = bincode::serialize(subject)
             .map_err(|e| format!("Can't serialize {}: {}", subject, e))?;
@@ -91,6 +97,7 @@ impl Db {
     }
 
     /// Search for a value, get a PropSubjectMap. If it does not exist, create a new one.
+    #[tracing::instrument(skip(self))]
     pub fn get_prop_subject_map(&self, string_val: &str) -> AtomicResult<PropSubjectMap> {
         let prop_sub_map = self
             .index_vals
@@ -187,6 +194,7 @@ impl Storelike for Db {
         Ok(())
     }
 
+    #[tracing::instrument(skip(self, resource), fields(sub = %resource.get_subject()))]
     fn add_resource_opts(
         &self,
         resource: &Resource,
@@ -223,6 +231,7 @@ impl Storelike for Db {
         self.set_propvals(resource.get_subject(), resource.get_propvals())
     }
 
+    #[tracing::instrument(skip(self))]
     fn remove_atom_from_index(&self, atom: &Atom) -> AtomicResult<()> {
         let vec = match atom.value.to_owned() {
             Value::ResourceArray(_v) => atom.values_to_subjects()?,
@@ -263,6 +272,7 @@ impl Storelike for Db {
         }
     }
 
+    #[tracing::instrument(skip(self))]
     fn get_resource(&self, subject: &str) -> AtomicResult<Resource> {
         let propvals = self.get_propvals(subject);
 
@@ -275,12 +285,14 @@ impl Storelike for Db {
         }
     }
 
+    #[tracing::instrument(skip(self))]
     fn get_resource_extended(
         &self,
         subject: &str,
         skip_dynamic: bool,
         for_agent: Option<&str>,
     ) -> AtomicResult<Resource> {
+        tracing::trace!("get_resource_extended: {}", subject);
         // This might add a trailing slash
         let mut url = url::Url::parse(subject)?;
         let clone = url.clone();
@@ -294,20 +306,22 @@ impl Storelike for Db {
         }
 
         // Check if the subject matches one of the endpoints
-        // TODO: do this on initialize, not on request!
-        let endpoints = crate::endpoints::default_endpoints();
-        let mut endpoint_resource = None;
-        endpoints.into_iter().for_each(|endpoint| {
+        for endpoint in self.endpoints.iter() {
             if url.path().starts_with(&endpoint.path) {
-                endpoint_resource = Some((endpoint.handle)(clone.clone(), self, for_agent))
+                // Not all Endpoitns have a hanlde function.
+                // If there is none, return the endpoint plainly.
+                let mut resource = if let Some(handle) = endpoint.handle {
+                    // Call the handle function for the endpoint, if it exists.
+                    (handle)(clone.clone(), self, for_agent).map_err(|e| {
+                        format!("Error handling {} Endpoint: {}", endpoint.shortname, e)
+                    })?
+                } else {
+                    endpoint.to_resource(self)?
+                };
+                // Extended resources must always return the requested subject as their own subject
+                resource.set_subject(subject.into());
+                return Ok(resource.to_owned());
             }
-        });
-
-        if let Some(resource) = endpoint_resource {
-            let mut resource_updated = resource?;
-            // Extended resources must always return the requested subject as their own subject
-            resource_updated.set_subject(subject.into());
-            return Ok(resource_updated);
         }
 
         let mut resource = self.get_resource(&removed_query_params)?;
@@ -316,12 +330,7 @@ impl Storelike for Db {
         resource.set_subject(subject.into());
 
         if let Some(agent) = for_agent {
-            if !crate::hierarchy::check_read(self, &resource, agent)? {
-                return Err(AtomicError::unauthorized(format!(
-                    "Agent '{}' is not authorized to read '{}'. There should be a `read` right in this resource or one of its parents.",
-                    agent, subject
-                )));
-            }
+            crate::hierarchy::check_read(self, &resource, agent)?;
         }
 
         // Whether the resource has dynamic properties
@@ -332,10 +341,11 @@ impl Storelike for Db {
                 crate::urls::COLLECTION => {
                     has_dynamic = true;
                     if !skip_dynamic {
-                        return crate::collections::construct_collection(
+                        return crate::collections::construct_collection_from_params(
                             self,
                             query_params,
                             &mut resource,
+                            for_agent,
                         );
                     }
                 }
@@ -371,6 +381,7 @@ impl Storelike for Db {
         Ok(resource)
     }
 
+    #[tracing::instrument(skip(self))]
     fn all_resources(&self, include_external: bool) -> ResourceCollection {
         let mut resources: ResourceCollection = Vec::new();
         let self_url = self
@@ -405,6 +416,7 @@ impl Storelike for Db {
         Ok(())
     }
 
+    #[tracing::instrument(skip(self))]
     fn remove_resource(&self, subject: &str) -> AtomicResult<()> {
         if let Ok(found) = self.get_propvals(subject) {
             for (prop, val) in found {
@@ -428,6 +440,7 @@ impl Storelike for Db {
     }
 
     // TPF implementation that used the index_value cache, far more performant than the StoreLike implementation
+    #[tracing::instrument(skip(self))]
     fn tpf(
         &self,
         q_subject: Option<&str>,
@@ -436,6 +449,7 @@ impl Storelike for Db {
         // Whether resources from outside the store should be searched through
         include_external: bool,
     ) -> AtomicResult<Vec<Atom>> {
+        tracing::trace!("tpf");
         let mut vec: Vec<Atom> = Vec::new();
 
         let hassub = q_subject.is_some();
