@@ -9,7 +9,7 @@ pub fn version_endpoint() -> Endpoint {
         params: [urls::SUBJECT.to_string()].into(),
         description: "Constructs a version of a resource from a Commit URL.".to_string(),
         shortname: "versions".to_string(),
-        handle: handle_version_request,
+        handle: Some(handle_version_request),
     }
 }
 
@@ -20,10 +20,11 @@ pub fn all_versions_endpoint() -> Endpoint {
         description: "Shows all versions for some resource. Constructs these using Commits."
             .to_string(),
         shortname: "all-versions".to_string(),
-        handle: handle_all_versions_request,
+        handle: Some(handle_all_versions_request),
     }
 }
 
+#[tracing::instrument(skip(store))]
 fn handle_version_request(
     url: url::Url,
     store: &impl Storelike,
@@ -45,6 +46,7 @@ fn handle_version_request(
     Ok(resource)
 }
 
+#[tracing::instrument(skip(store))]
 fn handle_all_versions_request(
     url: url::Url,
     store: &impl Storelike,
@@ -84,19 +86,39 @@ fn handle_all_versions_request(
     collection.to_resource(store)
 }
 
-/// Searches the local store for all commits with this subject
+/// Searches the local store for all commits with this subject, returns sorted from old to new.
+#[tracing::instrument(skip(store))]
 fn get_commits_for_resource(subject: &str, store: &impl Storelike) -> AtomicResult<Vec<Commit>> {
     let commit_atoms = store.tpf(None, Some(urls::SUBJECT), Some(subject), false)?;
     let mut commit_resources = Vec::new();
     for atom in commit_atoms {
+        // TODO: This will fail if a resource simply uses the SUBJECT url without being a valid Commit.
         let commit = crate::Commit::from_resource(store.get_resource(&atom.subject)?)?;
         commit_resources.push(commit)
     }
+    // Sort all commits by date
+    commit_resources.sort_by(|a, b| a.created_at.cmp(&b.created_at));
+
     Ok(commit_resources)
 }
 
+#[tracing::instrument(skip(store))]
+pub fn get_initial_commit_for_resource(
+    subject: &str,
+    store: &impl Storelike,
+) -> AtomicResult<Commit> {
+    let commits = get_commits_for_resource(subject, store)?;
+    if commits.is_empty() {
+        return Err(AtomicError::not_found(
+            "No commits found for this resource".to_string(),
+        ));
+    }
+    Ok(commits.first().unwrap().clone())
+}
+
 /// Constructs a Resource version for a specific Commit
 /// Only works if the current store has the required Commits
+#[tracing::instrument(skip(store))]
 pub fn construct_version(
     commit_url: &str,
     store: &impl Storelike,
@@ -107,16 +129,9 @@ pub fn construct_version(
     let subject = &commit.get(urls::SUBJECT)?.to_string();
     if let Some(agent) = &for_agent {
         let current_resource = store.get_resource(subject)?;
-        let can_open = crate::hierarchy::check_read(store, &current_resource, agent)?;
-        if !can_open {
-            return Err(AtomicError::unauthorized(
-                "You do not have permission to construct this resource".to_string(),
-            ));
-        }
+        crate::hierarchy::check_read(store, &current_resource, agent)?;
     }
-    let mut commits = get_commits_for_resource(subject, store)?;
-    // Sort all commits by date
-    commits.sort_by(|a, b| a.created_at.cmp(&b.created_at));
+    let commits = get_commits_for_resource(subject, store)?;
     let mut version = Resource::new(subject.into());
     for commit in commits {
         if let Some(current_commit) = commit.url.clone() {
