@@ -25,6 +25,38 @@ pub struct Resource {
 pub type PropVals = HashMap<String, Value>;
 
 impl Resource {
+    /// Adds a subject to some property vector / array.
+    /// Creates the vector if it does not exist.
+    pub fn append_subjects(
+        &mut self,
+        property: &str,
+        values: Vec<String>,
+        must_be_unique: bool,
+        store: &impl Storelike,
+    ) -> AtomicResult<()> {
+        // Does this property exist?
+        match self.get(property) {
+            Ok(existing) => {
+                let mut subjects = existing.to_subjects(None)?;
+                // Does it contain any of the passed values?
+                // Add them when it doesn't.
+                for value in values {
+                    if subjects.contains(&value) {
+                        if must_be_unique {
+                            subjects.push(value);
+                        }
+                        continue;
+                    } else {
+                        subjects.push(value);
+                        continue;
+                    }
+                }
+                self.set_propval(property.into(), subjects.into(), store)
+            }
+            Err(_no_val) => self.set_propval(property.into(), values.into(), store),
+        }
+    }
+
     /// Fetches all 'required' properties. Returns an error if any are missing in this Resource.
     pub fn check_required_props(&self, store: &impl Storelike) -> AtomicResult<()> {
         let classvec = self.get_classes(store)?;
@@ -42,7 +74,10 @@ impl Resource {
     }
 
     /// Removes / deletes the resource from the store by performing a Commit.
-    pub fn destroy(&mut self, store: &impl Storelike) -> AtomicResult<crate::Commit> {
+    pub fn destroy(
+        &mut self,
+        store: &impl Storelike,
+    ) -> AtomicResult<crate::commit::CommitResponse> {
         self.commit.destroy(true);
         self.save(store)
     }
@@ -215,7 +250,7 @@ impl Resource {
     /// Uses default Agent to sign the Commit.
     /// Stores changes on the Subject's Server by sending a Commit.
     /// Returns the generated Commit.
-    pub fn save(&mut self, store: &impl Storelike) -> AtomicResult<crate::Commit> {
+    pub fn save(&mut self, store: &impl Storelike) -> AtomicResult<crate::commit::CommitResponse> {
         let agent = store.get_default_agent()?;
         let commitbuilder = self.get_commit_builder().clone();
         let commit = commitbuilder.sign(&agent, store)?;
@@ -225,10 +260,10 @@ impl Resource {
             crate::client::post_commit(&commit, store)?;
         }
         // If that succeeds, save it locally;
-        commit.apply(store)?;
+        let commit_response = commit.apply(store)?;
         // then, reset the internal CommitBuiler.
         self.reset_commit_builder();
-        Ok(commit)
+        Ok(commit_response)
     }
 
     /// Saves the resource (with all the changes) to the store by creating a Commit.
