@@ -179,11 +179,11 @@ mod test {
         assert!(atoms.len() > 10);
         // Find by subject
         let atoms = store.tpf(Some(urls::CLASS), None, None, true).unwrap();
-        assert!(atoms.len() == 5);
+        assert_eq!(atoms.len(), 6);
         // Find by value
         let atoms = store.tpf(None, None, Some("class"), true).unwrap();
-        assert!(atoms[0].subject == urls::CLASS);
-        assert!(atoms.len() == 1);
+        assert_eq!(atoms[0].subject, urls::CLASS);
+        assert_eq!(atoms.len(), 1);
         // Find by property and value
         let atoms = store
             .tpf(None, Some(urls::SHORTNAME), Some("class"), true)
