@@ -173,10 +173,8 @@ pub trait Storelike: Sized {
         let _ignore = skip_dynamic;
         let resource = self.get_resource(subject)?;
         if let Some(agent) = for_agent {
-            if hierarchy::check_read(self, &resource, agent)? {
-                return Ok(resource);
-            }
-            return Err(AtomicError::unauthorized("No rights".into()));
+            hierarchy::check_read(self, &resource, agent)?;
+            return Ok(resource);
         }
         Ok(resource)
     }
@@ -184,7 +182,10 @@ pub trait Storelike: Sized {
     fn handle_not_found(&self, subject: &str, error: AtomicError) -> AtomicResult<Resource> {
         if let Some(self_url) = self.get_self_url() {
             if subject.starts_with(&self_url) {
-                return Err(format!("Failed to retrieve locally: '{}'. {}", subject, error).into());
+                return Err(AtomicError::not_found(format!(
+                    "Failed to retrieve locally: '{}'. {}",
+                    subject, error
+                )));
             }
         }
         self.fetch_resource(subject)
